local api = require("gemini.api")
local M = {}

-- State for managing current suggestion and debounce timer
local current_suggestion = {
    text = nil,
    start_col = nil,
    namespace_id = vim.api.nvim_create_namespace('gemini_suggestion'),
    timer = nil,
    last_request = 0,
    cache = {},
    debounce_ms = 1000, -- Wait 1 second between requests
}

-- Debug function
local function debug_print(...)
    vim.notify(string.format(...), vim.log.levels.INFO)
end

-- Helper function to clear suggestion
local function clear_suggestion()
    if current_suggestion.timer then
        vim.fn.timer_stop(current_suggestion.timer)
        current_suggestion.timer = nil
    end
    
    if current_suggestion.text then
        vim.api.nvim_buf_clear_namespace(0, current_suggestion.namespace_id, 0, -1)
        current_suggestion.text = nil
        current_suggestion.start_col = nil
    end
end

-- Check if we should rate limit
local function should_rate_limit()
    local now = vim.loop.now()
    if (now - current_suggestion.last_request) < current_suggestion.debounce_ms then
        return true
    end
    current_suggestion.last_request = now
    return false
end

-- Cache key generation
local function get_cache_key(context)
    return vim.fn.sha256(context)
end

-- Helper function to show suggestion
local function show_suggestion(suggestion, start_col)
    clear_suggestion()
    
    -- Split suggestion into lines
    local suggestion_lines = vim.split(suggestion, "\n")
    if #suggestion_lines == 0 then return end
    
    -- Get current line info
    local line = vim.api.nvim_win_get_cursor(0)[1] - 1
    local line_text = vim.api.nvim_buf_get_lines(0, line, line + 1, true)[1]
    
    -- Get text before cursor on current line
    local prefix = string.sub(line_text, 1, start_col)
    
    -- Process first line of suggestion
    local first_line = suggestion_lines[1]
    
    -- Find the longest common prefix between the input and suggestion
    local common_length = 0
    local input_after_cursor = string.sub(line_text, start_col + 1)
    local suggestion_text = first_line
    
    while common_length < #input_after_cursor and common_length < #suggestion_text do
        if string.sub(input_after_cursor, common_length + 1, common_length + 1) ==
           string.sub(suggestion_text, common_length + 1, common_length + 1) then
            common_length = common_length + 1
        else
            break
        end
    end
    
    -- Only show the part of suggestion that doesn't overlap with existing text
    first_line = string.sub(suggestion_text, common_length + 1)
    
    if first_line == "" and #suggestion_lines == 1 then return end
    
    current_suggestion.text = suggestion
    current_suggestion.start_col = start_col
    
    -- Show first line as virtual text only if there's non-overlapping content
    if first_line ~= "" then
        vim.api.nvim_buf_set_extmark(0, current_suggestion.namespace_id, line, start_col + common_length, {
            virt_text = {{first_line, 'GeminiSuggestion'}},
            virt_text_pos = 'inline',
            virt_text_hide = true,
        })
    end
    
    -- Show remaining lines as virtual lines
    if #suggestion_lines > 1 then
        local virt_lines = {}
        for i = 2, #suggestion_lines do
            table.insert(virt_lines, {{suggestion_lines[i], 'GeminiSuggestion'}})
        end
        
        vim.api.nvim_buf_set_extmark(0, current_suggestion.namespace_id, line, start_col + common_length, {
            virt_lines = virt_lines,
            virt_lines_above = false,
        })
    end
    
    debug_print("Showing suggestion (%d lines, common prefix length: %d)", #suggestion_lines, common_length)
end

function M.accept_suggestion()
    if not (current_suggestion.text and current_suggestion.start_col) then
        return false
    end

    local line = vim.api.nvim_win_get_cursor(0)[1] - 1
    local col = vim.api.nvim_win_get_cursor(0)[2]
    
    -- Only accept if we're still at or before the suggestion start
    if col < current_suggestion.start_col then
        return false
    end
    
    -- Get current line text
    local line_text = vim.api.nvim_buf_get_lines(0, line, line + 1, true)[1]
    local text_after_cursor = string.sub(line_text, col + 1)
    
    -- Split suggestion into lines
    local suggestion_lines = vim.split(current_suggestion.text, "\n")
    if #suggestion_lines == 0 then return false end
    
    -- For the first line, only insert the part that doesn't overlap with existing text
    local first_line = suggestion_lines[1]
    local common_length = 0
    while common_length < #text_after_cursor and common_length < #first_line do
        if string.sub(text_after_cursor, common_length + 1, common_length + 1) ==
           string.sub(first_line, common_length + 1, common_length + 1) then
            common_length = common_length + 1
        else
            break
        end
    end
    
    -- Only insert the non-overlapping part
    local to_insert = string.sub(first_line, common_length + 1)
    if to_insert ~= "" then
        vim.api.nvim_buf_set_text(
            0,
            line,
            col,
            line,
            col,
            {to_insert}
        )
    end
    
    -- Insert remaining lines below
    if #suggestion_lines > 1 then
        vim.api.nvim_buf_set_lines(
            0,
            line + 1,
            line + 1,
            false,
            vim.list_slice(suggestion_lines, 2)
        )
    end
    
    -- Move cursor to end of inserted text
    local final_line = line + #suggestion_lines - 1
    local final_col
    if final_line == line then
        final_col = col + #to_insert
    else
        final_col = #suggestion_lines[#suggestion_lines]
    end
    vim.api.nvim_win_set_cursor(0, {final_line + 1, final_col})
    
    clear_suggestion()
    return true
end

function M.trigger_completion()
    -- Clear any existing timer
    if current_suggestion.timer then
        vim.fn.timer_stop(current_suggestion.timer)
    end
    
    -- Set up debounce timer
    current_suggestion.timer = vim.fn.timer_start(300, function()
        -- Get current buffer and cursor info
        local cursor = vim.api.nvim_win_get_cursor(0)
        local line = cursor[1]
        local col = cursor[2]
        
        -- Get entire buffer content
        local lines = vim.api.nvim_buf_get_lines(0, 0, -1, false)
        local current_line = lines[line]
        
        -- Don't trigger completion if line is empty or cursor is at the start
        if not current_line or col == 0 or current_line:match("^%s*$") then
            clear_suggestion()
            return
        end
        
        -- Get text before cursor on current line
        local prefix = string.sub(current_line, 1, col)
        
        -- Don't trigger on certain conditions
        if prefix:match("^%s*$") or -- empty or whitespace
           prefix:match("[%s%-]$") or -- ends with space or dash
           #prefix < 3 then -- too short
            clear_suggestion()
            return
        end
        
        -- Construct context with line numbers and cursor position
        local context = {}
        local cursor_line = line
        
        -- Add up to 10 previous lines for context
        local start_line = math.max(1, line - 10)
        for i = start_line, line - 1 do
            if lines[i] and #lines[i] > 0 then
                table.insert(context, string.format("L%d: %s", i, lines[i]))
            end
        end
        
        -- Add current line with cursor position marker (^)
        if current_line and #current_line > 0 then
            local cursor_marker = string.rep(" ", col) .. "^"
            table.insert(context, string.format("L%d (current): %s", cursor_line, current_line))
            table.insert(context, cursor_marker)
        end
        
        -- Add up to 10 lines after current line
        local end_line = math.min(#lines, line + 10)
        for i = line + 1, end_line do
            if lines[i] and #lines[i] > 0 then
                table.insert(context, string.format("L%d: %s", i, lines[i]))
            end
        end
        
        -- Combine all lines and validate
        local full_context = table.concat(context, "\n")
        if #full_context == 0 then
            debug_print("Empty context, skipping completion request")
            return
        end
        
        -- Get file type for better context
        local file_type = vim.bo.filetype
        
        -- Construct prompt for Gemini
        local prompt = string.format([[I'm editing a %s file. Here's the context around line %d where my cursor (^) is:

%s

Complete the code at the cursor position. Consider the syntax and style of the surrounding code.
Return ONLY the completion that would naturally follow, no explanation.
The completion can be multiple lines if appropriate for the context.]], file_type, cursor_line, full_context)
        
        -- Check cache first
        local cache_key = get_cache_key(prompt)
        if current_suggestion.cache[cache_key] then
            debug_print("Using cached completion")
            show_suggestion(current_suggestion.cache[cache_key], col)
            return
        end
        
        -- Check rate limiting
        if should_rate_limit() then
            debug_print("Rate limited, skipping completion request")
            return
        end
        
        -- Get completion from Gemini
        api.get_response(prompt, nil, function(response, error)
            if error then 
                debug_print("Completion error: %s", error)
                return 
            end
            
            if type(response) == "string" and #response > 0 then
                -- Cache the response
                current_suggestion.cache[cache_key] = response
                
                vim.schedule(function()
                    -- Check if cursor position is still valid
                    local new_cursor = vim.api.nvim_win_get_cursor(0)
                    if new_cursor[1] == line and new_cursor[2] >= col then
                        show_suggestion(response, col)
                    end
                end)
            end
        end)
    end)
end

-- Setup function to create highlight group and keymaps
function M.setup()
    -- Create highlight group for suggestions
    vim.api.nvim_set_hl(0, 'GeminiSuggestion', {
        fg = '#666666',
        italic = true,
    })
    
    -- Map Tab to accept suggestion or behave normally
    vim.keymap.set('i', '<Tab>', function()
        if not M.accept_suggestion() then
            -- If no suggestion to accept, send regular Tab key
            vim.api.nvim_feedkeys(vim.api.nvim_replace_termcodes('<Tab>', true, true, true), 'n', true)
        end
    end, { expr = false, noremap = true })
    
    -- Set up autocommand for real-time completion
    vim.api.nvim_create_autocmd("TextChangedI", {
        pattern = "*",
        callback = function()
            M.trigger_completion()
        end
    })
    
    -- Clear suggestion on cursor move
    vim.api.nvim_create_autocmd({'CursorMovedI', 'CursorMoved'}, {
        pattern = "*",
        callback = function()
            clear_suggestion()
        end
    })
    
    debug_print("Gemini completion setup complete")
end

return M